@@ -1,9 +1,9 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-/* $Id: acconfig.h,v 1.6 2000/04/26 22:32:37 mrsam Exp $ */
+/* $Id: acconfig.h,v 1.7 2000/10/05 04:15:41 mrsam Exp $ */
 
 /* Whether we must provide a prototype for crypt() */
 #undef	NEED_CRYPT_PROTOTYPE
@@ -23,14 +23,5 @@
 /* Location of the userdb database */
 #undef	USERDB
 
-/* This macro is used to build the list of modules installed in libauth.a */
-#undef	STATIC_LIST
-
 /* This macro is used to build the list of SASL modules */
 #undef	SASL_LIST
-
-/* This macro is used to build the real list of modules, for authdaemond */
-#undef	DAEMON_STATIC_LIST
-
-/* The REAL modules, used by authdaemond */
-#undef	AUTHDAEMONMODULELIST
