@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -15,8 +15,9 @@
 #include	"auth.h"
 #include	"authmod.h"
 #include	"authldap.h"
+#include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authldap.c,v 1.10 2000/05/30 02:00:23 mrsam Exp $";
+static const char rcsid[]="$Id: authldap.c,v 1.12 2000/10/14 17:34:44 mrsam Exp $";
 
 
 struct callback_info {
@@ -250,3 +251,15 @@ char *auth_ldap(const char *service, const char *authtype, char *authdata,
 	return (0);
 #endif
 }
+
+
+extern int auth_ldap_pre(const char *userid, const char *service,
+        int (*callback)(struct authinfo *, void *),
+		  void *arg);
+
+struct authstaticinfo authldap_info={
+	"authldap",
+	auth_ldap,
+	auth_ldap_pre,
+	authldapclose,
+	0};
