@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -17,6 +17,7 @@
 #include	"auth.h"
 #include	"authmod.h"
 #include	"authwait.h"
+#include	"authstaticlist.h"
 
 #if	HAVE_SECURITY_PAM_APPL_H
 #include	<security/pam_appl.h>
@@ -26,7 +27,7 @@
 #include	<Pam/pam_appl.h>
 #endif
 
-static const char rcsid[]="$Id: authpam.c,v 1.9 2000/05/30 02:00:23 mrsam Exp $";
+static const char rcsid[]="$Id: authpam.c,v 1.11 2000/10/14 17:34:44 mrsam Exp $";
 
 static const char *pam_username, *pam_password, *pam_service;
 
@@ -322,3 +323,13 @@ int	rc;
 	return (ci.username);
 }
 
+static void auth_pam_cleanup()
+{
+}
+
+struct authstaticinfo authpam_info={
+	"authpam",
+	auth_pam,
+	auth_pam_pre,
+	auth_pam_cleanup,
+	0};
