@@ -15,7 +15,7 @@
 extern "C" {
 #endif
 
-static const char authstaticlist_h_rcsid[]="$Id: authstaticlist.h,v 1.5 2000/04/27 02:51:00 mrsam Exp $";
+static const char authstaticlist_h_rcsid[]="$Id: authstaticlist.h,v 1.6 2000/10/05 04:15:41 mrsam Exp $";
 
 struct authinfo;
 
@@ -32,7 +32,7 @@ struct authstaticinfo {
 			const char *);	/* NOT IMPLEMENTED YET */
 	} ;
 
-extern struct authstaticinfo authstaticlist[];
+extern struct authstaticinfo *authstaticmodulelist[];
 
 /*
 ** Call statically-linked authentication modules, according to the order
