@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -17,8 +17,9 @@
 
 #include	"auth.h"
 #include	"authmod.h"
+#include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authuserdb.c,v 1.12 2000/05/29 21:01:30 mrsam Exp $";
+static const char rcsid[]="$Id: authuserdb.c,v 1.13 2000/10/05 04:15:41 mrsam Exp $";
 
 extern int auth_userdb_pre_common(const char *, const char *, int,
         int (*callback)(struct authinfo *, void *),
@@ -133,3 +134,14 @@ struct	callback_info	ci;
         return (ci.userret);
 }
 
+extern int auth_userdb_pre(const char *userid, const char *service,
+        int (*callback)(struct authinfo *, void *),
+		    void *arg);
+
+struct authstaticinfo authuserdb_info={
+	"authuserdb",
+	auth_userdb,
+	auth_userdb_pre,
+	auth_userdb_cleanup,
+	0};
+
