@@ -16,8 +16,9 @@
 #endif
 #include	"auth.h"
 #include	"authmod.h"
+#include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authvchkpw.c,v 1.11 2000/04/30 01:04:09 mrsam Exp $";
+static const char rcsid[]="$Id: authvchkpw.c,v 1.14 2000/10/25 04:32:14 mrsam Exp $";
 
 extern int auth_vchkpw_pre(const char *userid, const char *service,
         int (*callback)(struct authinfo *, void *),
@@ -98,3 +99,15 @@ struct	callback_info	ci;
 
 	return (ci.userret);
 }
+
+static void authvchkpwclose()
+{
+}
+
+struct authstaticinfo authvchkpw_info={
+	"authvchkpw",
+	auth_vchkpw,
+	auth_vchkpw_pre,
+	authvchkpwclose,
+	0};
+
