@@ -25,7 +25,7 @@ extern char *authvchkpw_isvirtual(char *);
 extern FILE *authvchkpw_file(const char *, const char *);
 extern struct passwd *authvchkpw_search(FILE *, const char *);
 
-static const char rcsid[]="$Id: preauthvchkpw.c,v 1.10 2000/07/08 02:23:47 mrsam Exp $";
+static const char rcsid[]="$Id: preauthvchkpw.c,v 1.11 2000/10/06 23:45:17 mrsam Exp $";
 
 int auth_vchkpw_pre(const char *userid, const char *service,
         int (*callback)(struct authinfo *, void *),
@@ -40,12 +40,6 @@ int	notfound;
 struct authinfo auth;
 
 	memset(&auth, 0, sizeof(auth));
-	pw=getpwnam("vpopmail");
-	if (!pw)
-		return (-1);
-
-	uid=pw->pw_uid;
-	gid=pw->pw_gid;
 
 	usercopy=strdup(userid);
 	if (!usercopy)
@@ -70,6 +64,7 @@ struct authinfo auth;
 	}
 	
 
+	vget_assign(s,NULL,0,&uid, &gid);
 	pw=vauth_getpw(usercopy, s);
 	free(usercopy);
 	vclose();
