@@ -6,14 +6,6 @@
 #if	HAVE_CONFIG_H
 #include	"config.h"
 #endif
-#include	"imaptoken.h"
-#include	"imapwrite.h"
-#include	"imapscanclient.h"
-#include	"fetchinfo.h"
-#include	"rfc822/rfc822.h"
-#include	"rfc2045/rfc2045.h"
-#include	"maildir/maildirgetquota.h"
-#include	"maildir/maildirquota.h"
 
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -26,7 +18,16 @@
 #include	<sys/types.h>
 #include	<sys/stat.h>
 
-static const char rcsid[]="$Id: fetch.c,v 1.9 2000/08/20 17:22:35 mrsam Exp $";
+#include	"imaptoken.h"
+#include	"imapwrite.h"
+#include	"imapscanclient.h"
+#include	"fetchinfo.h"
+#include	"rfc822/rfc822.h"
+#include	"rfc2045/rfc2045.h"
+#include	"maildir/maildirgetquota.h"
+#include	"maildir/maildirquota.h"
+
+static const char rcsid[]="$Id: fetch.c,v 1.13 2000/11/15 17:52:21 mrsam Exp $";
 
 extern int current_mailbox_ro;
 extern struct imapscaninfo current_mailbox_info;
@@ -122,6 +123,9 @@ static void rfc822text(FILE *, struct fetchinfo *,
 	struct imapscaninfo *, unsigned,
 	struct rfc2045 *);
 
+static struct rfc2045 *fetch_alloc_rfc2045(unsigned, FILE *);
+static FILE *open_cached_fp(unsigned long);
+
 void fetchflags(unsigned long);
 
 static void fetcherrorprt(const char *p)
@@ -195,8 +199,11 @@ char	quotabuf[QUOTABUFSIZE];
 				rc= -1;
 			}
 			else
+			{
 				maildir_addquota(current_mailbox, quotafd,
 					quotabuf, nbytes, nmsgs);
+				if (quotafd >= 0)	close(quotafd);
+			}
 		}
 
 		if (rc == 0)
@@ -218,20 +225,20 @@ char	quotabuf[QUOTABUFSIZE];
 int do_fetch(unsigned long n, int byuid, void *p)
 {
 struct fetchinfo *fi=(struct fetchinfo *)p;
-int	fd;
 FILE	*fp;
 struct	rfc2045 *rfc2045p;
 int	seen;
 
-	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n-1);
-	if (fd < 0 || (fp=fdopen(fd, "r")) == 0)
+	fp=open_cached_fp(n-1);
+ 
+	if (!fp)
 	{
-		if (fd >= 0)	close(fd);
 		writes("* NO Cannot open message ");
 		writen(n);
 		writes("\r\n");
 		return (0);
 	}
+
 	writes("* ");
 	writen(n);
 	writes(" FETCH (");
@@ -260,8 +267,7 @@ int	seen;
 		if ((fi=fi->next) != 0)	writes(" ");
 	}
 	writes(")\r\n");
-	fclose(fp);
-	if (rfc2045p)	rfc2045_free(rfc2045p);
+
 	if (seen && !current_mailbox_ro)
 	{
 	struct	imapflags	flags;
@@ -272,11 +278,10 @@ int	seen;
 		{
 			flags.seen=1;
 			reflag_filename(&current_mailbox_info.msgs[n-1],&flags,
-				fd);
+				fileno(fp));
 			current_mailbox_info.msgs[n-1].changedflags=1;
 		}
 	}
-	close(fd);
 
 	if (current_mailbox_info.msgs[n-1].changedflags)
 		fetchflags(n-1);
@@ -370,13 +375,7 @@ int	rc=0;
 
 	if (parsemime && !*mimep)
 	{
-		if (fseek(fp, 0L, SEEK_SET) == -1)
-		{
-			fetcherror("fseek", fi, i, msgnum);
-			return (0);
-		}
-		*mimep=rfc2045_fromfp(fp);
-		if (!*mimep)	write_error_exit(0);
+		*mimep=fetch_alloc_rfc2045(msgnum, fp);
 	}
 
 	(*fetchfunc)(fp, fi, i, msgnum, *mimep);
@@ -591,6 +590,22 @@ unsigned bufptr;
 unsigned long skipping;
 int	ismsgrfc822=1;
 
+off_t start_seek_pos;
+
+/*
+** To optimize consecutive FETCHes, we cache our virtual and physical
+** position.  What we do is that on the first fetch we count off the
+** characters we read, and keep track of both the physical and the CRLF-based
+** offset into the message.  Then, on subsequent FETCHes, we attempt to
+** use that information.
+*/
+
+off_t cnt_virtual_chars;
+off_t cnt_phys_chars;
+
+off_t cache_virtual_chars;
+off_t cache_phys_chars;
+
 	while (p && isdigit((int)(unsigned char)*p))
 	{
 	unsigned n=0;
@@ -646,23 +661,12 @@ int	ismsgrfc822=1;
 
 	if (p && strcmp(p, "TEXT") == 0)
 	{
-		if (fseek(fp, start_body, SEEK_SET) == -1)
-		{
-			writes("{0}\r\n");
-			fetcherror("fseek", fi, i, msgnum);
-			return;
-		}
-
+		start_seek_pos=start_body;
 		cnt=end_pos - start_body + nbodylines;
 	}
 	else if (p && strcmp(p, "HEADER") == 0)
 	{
-		if (fseek(fp, start_pos, SEEK_SET) == -1)
-		{
-			writes("{0}\r\n");
-			fetcherror("fseek", fi, i, msgnum);
-			return;
-		}
+		start_seek_pos=start_pos;
 		cnt= start_body - start_pos + (nlines - nbodylines);
 	}
 	else if (p && strcmp(p, "HEADER.FIELDS") == 0)
@@ -697,22 +701,14 @@ int	ismsgrfc822=1;
 	}
 	else if (*fi->bodysection == 0)
 	{
-		if (fseek(fp, start_pos, SEEK_SET) == -1)
-		{
-			writes("{0}\r\n");
-			fetcherror("fseek", fi, i, msgnum);
-			return;
-		}
+		start_seek_pos=start_pos;
+
 		cnt= end_pos - start_pos + nlines;
 	}
 	else	/* Last possibility: entire body */
 	{
-		if (fseek(fp, start_body, SEEK_SET) == -1)
-		{
-			writes("{0}\r\n");
-			fetcherror("fseek", fi, i, msgnum);
-			return;
-		}
+		start_seek_pos=start_body;
+
 		cnt= end_pos - start_body + nbodylines;
 	}
 
@@ -726,6 +722,34 @@ int	ismsgrfc822=1;
 			cnt=fi->partialend;
 	}
 
+	if (get_cached_offsets(start_seek_pos, &cnt_virtual_chars,
+			       &cnt_phys_chars) == 0 &&
+	    cnt_virtual_chars <= skipping)	/* Yeah - cache it, baby! */
+	{
+		if (fseek(fp, start_seek_pos+cnt_phys_chars, SEEK_SET) == -1)
+		{
+			writes("{0}\r\n");
+			fetcherror("fseek", fi, i, msgnum);
+			return;
+		}
+		skipping -= cnt_virtual_chars;
+	}
+	else
+	{
+		if (fseek(fp, start_seek_pos, SEEK_SET) == -1)
+		{
+			writes("{0}\r\n");
+			fetcherror("fseek", fi, i, msgnum);
+			return;
+		}
+
+		cnt_virtual_chars=0;
+		cnt_phys_chars=0;
+	}
+
+	cache_virtual_chars=cnt_virtual_chars;
+	cache_phys_chars=cnt_phys_chars;
+
 	writes("{");
 	writen(cnt);
 	writes("}\r\n");
@@ -754,9 +778,12 @@ int	ismsgrfc822=1;
 
 		--rbufleft;
 		c=(int)(unsigned char)*rbufptr++;
+		++cnt_phys_chars;
 
 		if (c == '\n')
 		{
+			++cnt_virtual_chars;
+
 			if (skipping)
 				--skipping;
 			else
@@ -775,6 +802,7 @@ int	ismsgrfc822=1;
 			}
 		}
 
+		++cnt_virtual_chars;
 		if (skipping)
 			--skipping;
 		else
@@ -788,9 +816,13 @@ int	ismsgrfc822=1;
 			buf[bufptr++]=c;
 			--cnt;
 		}
+		cache_virtual_chars=cnt_virtual_chars;
+		cache_phys_chars=cnt_phys_chars;
 	}
 	writemem(buf, bufptr);
 	writeflush();
+	save_cached_offsets(start_seek_pos, cache_virtual_chars,
+			    cache_phys_chars);
 }
 
 static int dofetchheaderfields(struct fetchinfo *fi, const char *name)
@@ -866,7 +898,7 @@ int	ii;
 	}
 
 	ii=fread(buf, 1, start_body - start_pos, fp);
-	if (ii <= 0 || (i=ii) != start_body - start_pos)
+	if (ii < 0 || (i=ii) != start_body - start_pos)
 	{
 		fetcherror("unexpected EOF", fi, info, msgnum);
 		exit(1);
@@ -1312,3 +1344,121 @@ unsigned l;
 	}
 	writemem(buf, l);
 }
+
+/*
+** Poorly written IMAP clients (read: Netscape Messenger) like to issue
+** consecutive partial fetches for downloading large messages.
+**
+** To save the time of reparsing the MIME structure, we cache it.
+*/
+
+static struct rfc2045 *cached_rfc2045p;
+static char *cached_filename;
+
+static struct rfc2045 *fetch_alloc_rfc2045(unsigned msgnum, FILE *fp)
+{
+	if (cached_rfc2045p &&
+	    strcmp(cached_filename,
+		   current_mailbox_info.msgs[msgnum].filename) == 0)
+		return (cached_rfc2045p);
+
+	if (cached_rfc2045p)
+	{
+		rfc2045_free(cached_rfc2045p);
+		cached_rfc2045p=0;
+		free(cached_filename);
+		cached_filename=0;
+	}
+
+	if ((cached_filename=strdup(current_mailbox_info.
+				    msgs[msgnum].filename))
+	    == 0) write_error_exit(0);
+
+	if (fseek(fp, 0L, SEEK_SET) == -1)
+	{
+		write_error_exit(0);
+		return (0);
+	}
+	cached_rfc2045p=rfc2045_fromfp(fp);
+	if (!cached_rfc2045p)
+	{
+		free(cached_filename);
+		cached_filename=0;
+		write_error_exit(0);
+	}
+	return (cached_rfc2045p);
+}
+
+static FILE *cached_fp=0;
+static char *cached_fp_filename=0;
+static off_t cached_base_offset;
+static off_t cached_virtual_offset;
+static off_t cached_phys_offset;
+
+static FILE *open_cached_fp(unsigned long msgnum)
+{
+	int	fd;
+
+	if (cached_fp && strcmp(cached_fp_filename,
+				current_mailbox_info.msgs[msgnum].filename)
+	    == 0)
+		return (cached_fp);
+
+	if (cached_fp)
+	{
+		fclose(cached_fp);
+		free(cached_fp_filename);
+		cached_fp_filename=0;
+		cached_fp=0;
+	}
+
+	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, msgnum);
+	if (fd < 0 || (cached_fp=fdopen(fd, "r")) == 0)
+	{
+		if (fd >= 0)	close(fd);
+		return (0);
+	}
+
+	if ((cached_fp_filename=strdup(current_mailbox_info.
+				       msgs[msgnum].filename))
+	    == 0)
+	{
+		fclose(cached_fp);
+		cached_fp=0;
+		write_error_exit(0);
+	}
+	cached_base_offset=0;
+	cached_virtual_offset=0;
+	cached_phys_offset=0;
+	return (cached_fp);
+}
+
+void fetch_free_cache()
+{
+	if (cached_fp)
+	{
+		fclose(cached_fp);
+		cached_fp=0;
+		free(cached_fp_filename);
+		cached_fp_filename=0;
+	}
+}
+
+void save_cached_offsets(off_t base, off_t virt, off_t phys)
+{
+	cached_base_offset=base;
+	cached_virtual_offset=virt;
+	cached_phys_offset=phys;
+}
+
+int get_cached_offsets(off_t base, off_t *virt, off_t *phys)
+{
+	if (!cached_fp)
+		return (-1);
+	if (base != cached_base_offset)
+		return (-1);
+
+	*virt=cached_virtual_offset;
+	*phys=cached_phys_offset;
+	return (0);
+}
