@@ -72,7 +72,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.47 2000/09/29 04:38:47 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.50 2000/11/15 17:52:21 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -102,15 +102,20 @@ void rfc2045_error(const char *p)
 
 int is_trash(const char *m)
 {
-	if (strncmp(m, "." TRASH, sizeof("." TRASH)-1))	return (0);
-	switch (m[sizeof("." TRASH)-1])	{
-	case 0:
-	case '.':
+	if (strcmp(m, "." TRASH))
+	{
+		/*
+		 * not trying to delete .Trash but folder inside of .Trash
+		 */
+		return (0);
+	}
+	else
+	{
+		/*
+		 * trying to delete .Trash - stop them
+		 */
 		return (1);
-	default:
-		break;
 	}
-	return (0);
 }
 
 void emptytrash()
@@ -621,6 +626,7 @@ int	lastnl;
 			return (-1);
 		}
 		maildir_addquota(mailbox, quotafd, quotabuf, nbytes, 1);
+		if (quotafd >= 0)	close(quotafd);
 	}
 
 	rc=rename(tmpname, p);
@@ -943,6 +949,8 @@ struct imapflags flags;
 char	*old_name;
 int	move_to_trash=0;
 
+	fetch_free_cache();
+
 	old_name=getenv("IMAP_MOVE_EXPUNGE_TO_TRASH");
 	if (old_name && atoi(old_name) && !is_trash(current_mailbox))
 		move_to_trash=1;
@@ -1121,6 +1129,7 @@ int	uid=0;
 	if (strcmp(curtoken->tokenbuf, "LOGOUT") == 0)
 	{
 		if (nexttoken()->tokentype != IT_EOL)	return (-1);
+		fetch_free_cache();
 		writes("* BYE Courier-IMAP server shutting down\r\n");
 		writes(tag);
 		writes(" OK LOGOUT completed\r\n");
@@ -1543,7 +1552,7 @@ int	uid=0;
 			return (0);
 		}
 
-
+		fetch_free_cache();
 		if (strcmp(mailbox, ".") == 0 ||
 			is_reserved(mailbox) || mddelete(mailbox))
 		{
@@ -1625,6 +1634,7 @@ int	uid=0;
 			return (0);
 		}
 
+		fetch_free_cache();
 		if (strcmp(mailbox, "."))
 		{
 			if (is_reserved(mailbox) ||
@@ -1829,6 +1839,7 @@ int	uid=0;
 			return (0);
 		}
 
+		fetch_free_cache();
 		if (!dir || maildir_shared_unsubscribe(0, mailbox+7))
 		{
 			free(mailbox);
@@ -1958,6 +1969,7 @@ int	uid=0;
 			return (0);
 		}
 
+		fetch_free_cache();
 		if (do_msgset(msgset, &do_store, &storeinfo_s, uid))
 		{
 			writes(tag);
@@ -2360,6 +2372,7 @@ int	uid=0;
 			return (0);
 		}
 
+		fetch_free_cache();
 		cqinfo.destmailbox=mailbox;
 
 		/*
@@ -2410,6 +2423,7 @@ int	uid=0;
 
 			maildir_addquota(mailbox, quotafd, quotabuf,
 				cqinfo.nbytes, cqinfo.nfiles);
+			if (quotafd >= 0)	close(quotafd);
 		}
 
 		if (has_quota < 0 ||
