@@ -37,7 +37,7 @@
 #endif
 #endif
 
-static const char rcsid[]="$Id: storeinfo.c,v 1.6 2000/05/28 00:11:12 mrsam Exp $";
+static const char rcsid[]="$Id: storeinfo.c,v 1.7 2000/08/25 01:11:14 mrsam Exp $";
 
 extern int is_trash(const char *);
 extern int get_flagname(const char *, struct imapflags *);
@@ -276,7 +276,7 @@ int	fd;
 			close(fd);
 			return (0);
 		}
-		n=stat_buf.st_size;
+		nbytes=stat_buf.st_size;
 	}
 	close(fd);
 
