@@ -3,7 +3,7 @@
 ** See COPYING for distribution information.
 */
 
-static const char rcsid[]="$Id: thread.c,v 1.5 2000/09/02 23:55:55 mrsam Exp $";
+static const char rcsid[]="$Id: thread.c,v 1.6 2000/09/29 04:38:47 mrsam Exp $";
 
 #include	"config.h"
 #include	"imapd.h"
@@ -117,12 +117,12 @@ size_t	i;
 }
 
 void dothreadorderedsubj(struct searchinfo *si, struct searchinfo *sihead,
-	int isuid)
+			 const struct unicode_info *mycharset, int isuid)
 {
 struct	os_struct	os;
 
 	os_init(&os);
-	search_internal(si, sihead, isuid, thread_os_callback, &os);
+	search_internal(si, sihead, mycharset, isuid, thread_os_callback, &os);
 
 	if (os.nmsgs > 0)	/* Found some messages */
 	{
@@ -169,6 +169,7 @@ static void thread_os_callback(struct searchinfo *si,
 static void printthread(struct imap_refmsg *, int);
 
 void dothreadreferences(struct searchinfo *si, struct searchinfo *sihead,
+			const struct unicode_info *mycharset,
 			int isuid)
 {
 	struct imap_refmsgtable *reftable;
@@ -180,7 +181,8 @@ void dothreadreferences(struct searchinfo *si, struct searchinfo *sihead,
 		return;
 	}
 
-	search_internal(si, sihead, 0, thread_ref_callback, reftable);
+	search_internal(si, sihead, mycharset, 0,
+			thread_ref_callback, reftable);
 
 	root=rfc822_thread(reftable);
 	printthread(root, isuid);
@@ -321,7 +323,8 @@ size_t	i;
 	return (0);
 }
 
-void dosortmsgs(struct searchinfo *si, struct searchinfo *sihead, int isuid)
+void dosortmsgs(struct searchinfo *si, struct searchinfo *sihead,
+		const struct unicode_info *mycharset, int isuid)
 {
 struct	sortmsgs sm;
 struct	searchinfo *p;
@@ -341,7 +344,7 @@ struct	searchinfo *p;
 		default:
 			break;
 		}
-	search_internal(si, sihead, isuid, sort_callback, &sm);
+	search_internal(si, sihead, mycharset, isuid, sort_callback, &sm);
 	if (sm.nmsgs >= 0)
 	{
 	size_t	i;
@@ -421,7 +424,7 @@ size_t	i;
 			if (!p)	p="";
 			msg->sortfields[i]=my_strdup(p);
 			msg->sortorder[i]=rev;
-/* fprintf(stderr, "%d %s\n", msg->sortorder[i], msg->sortfields[i]); */
+			/* fprintf(stderr, "%d %s\n", msg->sortorder[i], msg->sortfields[i]); */
 			++i;
 			rev=0;
 			continue;
