@@ -21,6 +21,7 @@
 #include	<fcntl.h>
 #include	<signal.h>
 #include	"rfc2045.h"
+#include	"rfc822/rfc2047.h"
 #include	"rfc2045charset.h"
 #if HAVE_UNISTD_H
 #include	<unistd.h>
@@ -36,7 +37,7 @@ int gethostname(const char *, size_t);
 #endif
 
 
-static const char rcsid[]="$Id: reformime.c,v 1.29 2000/07/27 22:09:45 mrsam Exp $";
+static const char rcsid[]="$Id: reformime.c,v 1.32 2000/10/23 02:22:40 mrsam Exp $";
 
 void rfc2045_error(const char *errmsg)
 {
@@ -172,20 +173,62 @@ const char *p;
 	content_name=rfc2045_contentname(s);
 	printf("content-type: %s\n", content_type);
 	if (content_name)
-		printf("content-name: %s\n", content_name);
+	{
+	char *s=rfc2047_decode_simple(content_name);
+
+		if (!s)
+		{
+			perror("rfc2047_decode_simple");
+			exit(1);
+		}
+		printf("content-name: %s\n", s);
+		free(s);
+	}
+
 	printf("content-transfer-encoding: %s\n", transfer_encoding);
 	printf("charset: %s\n", charset);
 	if (disposition && *disposition)
 		printf("content-disposition: %s\n", disposition);
 	if (disposition_name && *disposition_name)
-		printf("content-disposition-name: %s\n", disposition_name);
+	{
+	char *s=rfc2047_decode_simple(disposition_name);
+
+		if (!s)
+		{
+			perror("rfc2047_decode_simple");
+			exit(1);
+		}
+		printf("content-disposition-name: %s\n", s);
+		free(s);
+	}
+
 	if (disposition_filename && *disposition_filename)
-		printf("content-disposition-filename: %s\n",
-				disposition_filename);
+	{
+	char *s=rfc2047_decode_simple(disposition_filename);
+
+		if (!s)
+		{
+			perror("rfc2047_decode_simple");
+			exit(1);
+		}
+		printf("content-disposition-filename: %s\n", s);
+		free(s);
+	}
+
 	if (*(p=rfc2045_content_id(s)))
 		printf("content-id: <%s>\n", p);
 	if (*(p=rfc2045_content_description(s)))
-		printf("content-description: %s\n", p);
+	{
+	char *s=rfc2047_decode_simple(p);
+
+		if (!s)
+		{
+			perror("rfc2047_decode_simple");
+			exit(1);
+		}
+		printf("content-description: %s\n", s);
+		free(s);
+	}
 	if (*(p=rfc2045_content_language(s)))
 		printf("content-language: %s\n", p);
 	if (*(p=rfc2045_content_md5(s)))
@@ -287,6 +330,7 @@ static char *get_suitable_filename(struct rfc2045 *r, const char *pfix,
 const char *disposition_s;
 const char *disposition_name_s;
 const char *disposition_filename_s;
+char	*filename_buf;
 const char *content_name_s;
 char	*p, *q;
 char	*dyn_disp_name=0;
@@ -301,6 +345,24 @@ char	*dyn_disp_name=0;
 	if (!disposition_filename_s || !*disposition_filename_s)
 		disposition_filename_s=content_name_s;
 
+	filename_buf=rfc2047_decode_simple(disposition_filename_s);
+
+	if (!filename_buf)
+	{
+		perror("rfc2047_decode_simple");
+		exit(1);
+	}
+
+	if (strlen(filename_buf) > 32)
+	{
+		p=filename_buf;
+		q=filename_buf + strlen(filename_buf)-32;
+		while ( (*p++ = *q++) != 0)
+			;
+	}
+
+	disposition_filename_s=filename_buf;
+
 	if (ignore_filename)
 	{
 	char	numbuf[NUMBUFSIZE];
@@ -372,6 +434,7 @@ char	*dyn_disp_name=0;
 		{
 			free(p);
 			fclose(tty);
+			free(filename_buf);
 			return (0);
 		}
 		fprintf (tty, "Filename [%s]: ", p);
@@ -395,6 +458,7 @@ char	*dyn_disp_name=0;
 			}
 		}
 	}
+	free(filename_buf);
 	return (p);
 }
 
