@@ -4,7 +4,7 @@
 */
 
 /*
-** $Id: imaprefs.c,v 1.1 2000/09/02 00:48:57 mrsam Exp $
+** $Id: imaprefs.c,v 1.4 2000/11/14 05:15:31 mrsam Exp $
 */
 
 #include	"config.h"
@@ -230,6 +230,7 @@ static void breakparent(struct imap_refmsg *m)
 static struct imap_refmsg *dorefcreate(struct imap_refmsgtable *mt,
 				       const char *newmsgid,
 				       struct rfc822a *a)
+     /* a - references header */
 {
 struct imap_refmsg *lastmsg=0, *m;
 struct imap_refmsg *msg;
@@ -318,6 +319,16 @@ int n;
 
 	msg=*newmsgid ? rfc822_threadsearchmsg(mt, newmsgid):0;
 
+	/*
+	       If a message does not contain a Message-ID header line,
+	       or the Message-ID header line does not contain a valid
+	       Message ID, then assign a unique Message ID to this
+	       message.
+
+	       Implementation note: empty msgid, plus dupe check below,
+	       implements that.
+	*/
+
 	if (msg && msg->isdummy)
 	{
 		msg->isdummy=0;
@@ -326,6 +337,27 @@ int n;
 	}
 	else
 	{
+#if 1
+		/*
+		** If two or more messages have the same Message ID, assign
+		** a unique Message ID to each of the duplicates.
+		**
+		** Implementation note: just unlink the existing message from
+		** it's parents/children.
+		*/
+		if (msg)
+		{
+			while (msg->firstchild)
+				breakparent(msg->firstchild);
+			breakparent(msg);
+			newmsgid="";
+
+			/* Create new entry with an empty msgid, if any more
+			** msgids come, they'll hit the dupe check again.
+			*/
+
+		}
+#endif
 		msg=rfc822_threadallocmsg(mt, newmsgid);
 		if (!msg)	return (0);
 	}
@@ -430,7 +462,12 @@ struct imap_refmsg *rfc822_threadgetroot(struct imap_refmsgtable *mt)
 
 	for (m=mt->firstmsg; m; m=m->next)
 		if (!m->parent)
+		{
+			if (m->isdummy && m->firstchild == 0)
+				continue; /* Can happen in reference creation */
+
 			linkparent(m, root);
+		}
 	root->parent=NULL;
 	return (mt->rootptr=root);
 }
@@ -528,50 +565,69 @@ int rfc822_threadsortsubj(struct imap_refmsgtable *mt,
 
 		subj=p->subj ? p->subj:"";
 
+
 		/*
-		** (ii) Lookup the message associated with this extracted
+		** (ii) If the extracted subject is empty, skip this
+		** message.
+		*/
+
+		if (*subj == 0)
+			continue;
+
+		/*
+		** (iii) Lookup the message associated with this extracted
 		** subject in the table.
-		**
-		** (iii) If there is no message in the table with this
-		** subject, add the current message and the extracted
-		** subject to the subject table.
 		*/
 
 		if (findsubj(mt, subj, &isrefwd, 1, &subjtop))
 			return (-1);
 
+		/*
+		**
+		** (iv) If there is no message in the table with this
+		** subject, add the current message and the extracted
+		** subject to the subject table.
+		*/
+
 		if (subjtop->msg == 0)
 		{
 			subjtop->msg=toproot;
 			subjtop->msgisrefwd=isrefwd;
 			continue;
 		}
+
 		/*
 		** Otherwise, replace the message in the table with the
-		** current message if either of the following criteria are
-		** true:
-		**
-		** The current message is a dummy and the one in the
-		** table is not, OR
+		** current message if the message in the table is not a
+		** dummy AND either of the following criteria are true:
 		*/
 
-		if (toproot->isdummy && !subjtop->msg->isdummy)
+		if (!subjtop->msg->isdummy)
 		{
-			subjtop->msg=toproot;
-			subjtop->msgisrefwd=isrefwd;
-			continue;
-		}
+			/*
+			** The current message is a dummy
+			**
+			*/
 
-		/*
-		** The message in the table is a reply or forward (its
-		** original subject contains a subj-refwd part and/or a
-		** "(fwd)" subj-trailer) and the current message is not.
-		*/
+			if (toproot->isdummy)
+			{
+				subjtop->msg=toproot;
+				subjtop->msgisrefwd=isrefwd;
+				continue;
+			}
 
-		if (subjtop->msgisrefwd && !isrefwd)
-		{
-			subjtop->msg=toproot;
-			subjtop->msgisrefwd=isrefwd;
+			/*
+			** The message in the table is a reply or forward (its
+			** original subject contains a subj-refwd part and/or a
+			** "(fwd)" subj-trailer) and the current message is
+			not.
+			*/
+
+			if (subjtop->msgisrefwd && !isrefwd)
+			{
+				subjtop->msg=toproot;
+				subjtop->msgisrefwd=isrefwd;
+			}
 		}
 	}
 	return (0);
@@ -608,7 +664,15 @@ int rfc822_threadmergesubj(struct imap_refmsgtable *mt,
 		subj=p->subj ? p->subj:"";
 
 		/*
-		** (ii) Lookup the message associated with this extracted
+		** (ii) If the extracted subject is empty, skip this
+		** message.
+		*/
+
+		if (*subj == 0)
+			continue;
+
+		/*
+		** (iii) Lookup the message associated with this extracted
 		** subject in the table.
 		*/
 
@@ -616,7 +680,7 @@ int rfc822_threadmergesubj(struct imap_refmsgtable *mt,
 			return (-1);
 
 		/*
-		** (iii) If the message in the table is the current message,
+		** (iv) If the message in the table is the current message,
 		** skip it.
 		*/
 
@@ -727,12 +791,16 @@ int rfc822_threadmergesubj(struct imap_refmsgtable *mt,
 }
 
 /*
+**
 ** (5) Traverse the messages under the root and sort each set of
-** siblings by date.  In the case of an exact match on date, use
-** the order in which they appear in the mailbox (that is, by
-** sequence number) to determine the order.  In the case of a
-** dummy message (which can occur with top-level siblings), use
-** the first child for sorting.
+** siblings by sent date.  Traverse the messages in such a way
+** that the "youngest" set of siblings are sorted first, and the
+** "oldest" set of siblings are sorted last (grandchildren are
+** sorted before children, etc).  In the case of an exact match on
+** sent date, use the order in which the messages appear in the
+** mailbox (that is, by sequence number) to determine the order.
+** In the case of a dummy message (which can only occur with top-
+** level siblings), use its first child for sorting.
 */
 
 static int cmp_msgs(const void *a, const void *b)
@@ -765,53 +833,70 @@ static int cmp_msgs(const void *a, const void *b)
 		na < nb ? -1: na > nb ? 1:0);
 }
 
+struct imap_threadsortinfo {
+	struct imap_refmsgtable *mt;
+	struct imap_refmsg **sort_table;
+	size_t sort_table_cnt;
+} ;
+
+static int dothreadsort(struct imap_threadsortinfo *,
+			struct imap_refmsg *);
+
 int rfc822_threadsortbydate(struct imap_refmsgtable *mt)
 {
-	struct imap_refmsg **sort_table=0;
-	size_t sort_table_cnt=0;
+	struct imap_threadsortinfo itsi;
+	int rc;
 
-	struct imap_refmsg *p, *q;
+	itsi.mt=mt;
+	itsi.sort_table=0;
+	itsi.sort_table_cnt=0;
 
-	for (p=mt->firstmsg; p; p=p->next)
+	rc=dothreadsort(&itsi, mt->rootptr);
+
+	if (itsi.sort_table)
+		free(itsi.sort_table);
+	return (rc);
+}
+
+static int dothreadsort(struct imap_threadsortinfo *itsi,
+			struct imap_refmsg *p)
+{
+	struct imap_refmsg *q;
+	size_t i, n;
+
+	for (q=p->firstchild; q; q=q->nextsib)
+		dothreadsort(itsi, q);
+
+	n=0;
+	for (q=p->firstchild; q; q=q->nextsib)
+		++n;
+
+	if (n > itsi->sort_table_cnt)
 	{
-		size_t i, n;
+		struct imap_refmsg **new_array=(struct imap_refmsg **)
+			(itsi->sort_table ?
+			 realloc(itsi->sort_table,
+				 sizeof(struct imap_refmsg *)*n)
+			 :malloc(sizeof(struct imap_refmsg *)*n));
 
-		if (!p->firstchild || !p->firstchild->nextsib)
-			continue;
+		if (!new_array)
+			return (-1);
 
-		n=0;
-		for (q=p->firstchild; q; q=q->nextsib)
-			++n;
-
-		if (n > sort_table_cnt)
-		{
-			struct imap_refmsg **new_array=(struct imap_refmsg **)
-				(sort_table ?
-				 realloc(sort_table, sizeof(struct imap_refmsg *)*n)
-				 :malloc(sizeof(struct imap_refmsg *)*n));
-
-			if (!new_array)
-			{
-				if (sort_table)
-					free(sort_table);
-				return (-1);
-			}
-			sort_table=new_array;
-			sort_table_cnt=n;
-		}
-		n=0;
-		while ((q=p->firstchild) != 0)
-		{
-			breakparent(q);
-			sort_table[n++]=q;
-		}
-		qsort(sort_table, n, sizeof(struct imap_refmsg *), cmp_msgs);
-
-		for (i=0; i<n; i++)
-			linkparent(sort_table[i], p);
+		itsi->sort_table=new_array;
+		itsi->sort_table_cnt=n;
 	}
-	if (sort_table)
-		free(sort_table);
+
+	n=0;
+	while ((q=p->firstchild) != 0)
+	{
+		breakparent(q);
+		itsi->sort_table[n++]=q;
+	}
+
+	qsort(itsi->sort_table, n, sizeof(struct imap_refmsg *), cmp_msgs);
+
+	for (i=0; i<n; i++)
+		linkparent(itsi->sort_table[i], p);
 	return (0);
 }
 
