@@ -12,7 +12,7 @@
 #include	"rfc822.h"
 #include	"rfc2047.h"
 
-static const char rcsid[]="$Id: rfc2047.c,v 1.3 2000/09/04 15:23:35 mrsam Exp $";
+static const char rcsid[]="$Id: rfc2047.c,v 1.4 2000/09/24 01:16:20 mrsam Exp $";
 
 static const char xdigit[]="0123456789ABCDEF";
 
@@ -147,6 +147,8 @@ char	*enctext;
 
 			for (q=r=enctext; *q; )
 			{
+				int c;
+
 				if (*q == '=' && q[1] && q[2])
 				{
 					*r++ = (char)(
@@ -154,7 +156,11 @@ char	*enctext;
 					q += 3;
 					continue;
 				}
-				*r++ = *q++ ;
+
+				c=*q++;
+				if (c == '_')
+					c=' ';
+				*r++ = c ;
 			}
 			*r=0;
 		}
