@@ -46,6 +46,8 @@
 #endif
 #include	"waitlib/waitlib.h"
 #include	"rfc1035/rfc1035.h"
+#include	"liblock/config.h"
+#include	"liblock/liblock.h"
 #include	"tcpremoteinfo.h"
 #include	"authlib/auth.h"
 #include	"numlib/numlib.h"
@@ -53,7 +55,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.17 2000/07/09 16:31:24 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.19 2000/09/17 18:19:05 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *listenarg=0;
@@ -67,9 +69,12 @@ static const char *nodnslookup=0;
 static const char *noidentlookup=0;
 static const char *stderrarg=0;
 static const char *stderrloggerarg=0;
-static const char *forcebindarg=0;
 static const char *pidarg=0;
 static const char *proxyarg=0;
+static const char *restartarg=0;
+static const char *stoparg=0;
+
+static char *lockfilename;
 
 static void setup_block(const char *);
 
@@ -77,7 +82,6 @@ static struct args arginfo[]={
 	{"access", &accessarg},
 	{"address", &ipaddrarg},
 	{"block", 0, setup_block},
-	{"forcebind", &forcebindarg},
 	{"group", &grouparg},
 	{"listen", &listenarg},
 	{"maxperc", &maxpercarg},
@@ -86,8 +90,10 @@ static struct args arginfo[]={
 	{"nodnslookup", &nodnslookup},
 	{"noidentlookup", &noidentlookup},
 	{"pid", &pidarg},
+	{"restart", &restartarg},
 	{"stderr", &stderrarg},
 	{"stderrlogger", &stderrloggerarg},
+	{"stop", &stoparg},
 	{"user", &userarg},
 	{"proxy", &proxyarg},
 	{0}
@@ -158,50 +164,9 @@ static int isid(const char *p)
 	return (1);
 }
 
-static void savepid()
-{
-FILE	*fp;
-char	pbuf[NUMBUFSIZE+2];
-
-	if (!pidarg)	return;
-
-	if ((fp=fopen(pidarg, "w")) != 0)
-	{
-		fprintf(fp, "%s\n", str_pid_t(getpid(), pbuf));
-		fclose(fp);
-	}
-}
-
-static void erasepid()
-{
-unsigned long n;
-char	buf[128];
-int	fd;
-int	i;
-
-	if (!pidarg)	return;
-
-	if ((fd=open(pidarg, O_RDONLY)) < 0)	return;
-	if ((i=sox_read(fd, buf, sizeof(buf)-1)) <= 0)
-	{
-		sox_close(fd);
-		return;
-	}
-	buf[i]=0;
-	n=0;
-	sox_close(fd);
-	for (i=0; buf[i]; i++)
-		if (buf[i] >= '0' && buf[i] <= '9')
-			n=n*10 + buf[i]-'0';
-
-	if (n == getpid())
-		unlink(pidarg);
-}
-
 static RETSIGTYPE sigexit(int n)
 {
 	kill( -getpid(), SIGTERM);
-	erasepid();
 	_exit(0);
 
 #if RETSIGTYPE != void
@@ -235,19 +200,42 @@ struct sockaddr_in6	sin6;
 struct sockaddr_in	sin4;
 
 int	af;
-int	rc;
 struct	group *gr;
 int	i;
 gid_t	gid=0;
 const	char *servname;
 struct servent *servptr;
 int	forced=0;
-
-	for (i=3; i<256; i++)
-		close(i);
-
+int	lockfd;
+ 
 	argn=argparse(argc, argv, arginfo);
 
+	if (pidarg == 0)
+	{
+		fprintf(stderr, "%s: -pid argument is required.\n", argv[0]);
+		return (-1);
+	}
+
+	lockfilename=malloc(strlen(pidarg)+sizeof(".lock"));
+	if (!lockfilename)
+	{
+		perror("malloc");
+		return (-1);
+	}
+	strcat(strcpy(lockfilename, pidarg), ".lock");
+
+	if (stoparg)
+	{
+		ll_daemon_stop(lockfilename, pidarg);
+		exit(0);
+	}
+
+	if (restartarg)
+	{
+		ll_daemon_restart(lockfilename, pidarg);
+		exit(0);
+	}
+
 	if (argc - argn < 2)
 	{
 		fprintf(stderr, "Usage: %s [options] port prog arg1 arg2...\n",
@@ -255,6 +243,13 @@ int	forced=0;
 		return (-1);
 	}
 
+	lockfd=ll_daemon_start(lockfilename);
+	if (lockfd < 0)
+	{
+		perror("ll_daemon_start");
+		return (-1);
+	}
+
 	servname=argv[argn++];
 
 	servptr=getservbyname(servname, "tcp");
@@ -276,6 +271,7 @@ int	forced=0;
 	if ((fd=rfc1035_mksocket(SOCK_STREAM, 0, &af)) < 0)
 	{
 		perror("socket");
+		close(lockfd);
 		return (-1);
 	}
 
@@ -287,6 +283,7 @@ int	forced=0;
 		{
 			fprintf(stderr,"Invalid IP address: %s\n", ipaddrarg);
 			close(fd);
+			close(lockfd);
 			return (-1);
 		}
 
@@ -296,6 +293,7 @@ int	forced=0;
 			fprintf(stderr,"Unable to bind IP address: %s\n",
 				ipaddrarg);
 			close(fd);
+			close(lockfd);
 			return (-1);
 		}
 	}
@@ -325,15 +323,37 @@ int	forced=0;
 		else
 		{
 			errno=EAFNOSUPPORT;
-			rc= -1;
+			perror("socket");
+			close(fd);
+			close(lockfd);
+			return (-1);
 		}
 	}
 
+	{
+		int dummy=1;
+
+		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+		       (const char *)&dummy, sizeof(dummy)) < 0)
+		{
+			perror("setsockopt");
+		}
+	}
+
+
+	if (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
+	{
+		perror("bind");
+		close(fd);
+		close(lockfd);
+		return (-1);
+	}
+
 	signal(SIGINT, sigexit);
 	signal(SIGHUP, sighup);
 	signal(SIGTERM, sigexit);
 
-	while (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
+#if 0
 	{
 	int	fd2;
 	int	dummy;
@@ -367,8 +387,10 @@ int	forced=0;
 		sleep(60);
 		forced=1;
 	}
+#endif
+
+	ll_daemon_started(pidarg, lockfd);
 
-	savepid();
 	if (grouparg)
 	{
 		if (isid(grouparg))
@@ -377,6 +399,7 @@ int	forced=0;
 		{
 			fprintf(stderr, "Group not found: %s\n", grouparg);
 			sox_close(fd);
+			close(lockfd);
 			return (-1);
 		}
 		else	gid=gr->gr_gid;
@@ -402,6 +425,14 @@ int	forced=0;
 		}
 	}
 
+	if (ll_daemon_resetio())
+	{
+		perror("ll_daemon_resetio");
+		close(fd);
+		close(lockfd);
+		return (-1);
+	}
+
 	if (stderrloggerarg)
 	{
 	pid_t	p;
@@ -1291,13 +1322,11 @@ int rc;
 
 	if (argn < 0)
 	{
-		erasepid();
 		exit(1);
 	}
 	if (accessarg && openaccess(accessarg))
 		perror(accessarg);
 	rc=doit(argn, argc, argv);
-	erasepid();
 	kill( -getpid(), SIGTERM);
 	exit(rc);
 	return (0);
